/** operation **/
package jsparky;

import org.w3c.dom.*;
import jsparky.operations.*;

public abstract class Operation {

    /** operation names constants **/
    // first group
    public static final String MERGE = "MERGE";
    public static final String RELATIVE_CLAUSE = "RELATIVE_CLAUSE";
    public static final String WITH_NS = "WITH_NS";
    public static final String WITH_SN = "WITH_SN";
    // second group
    public static final String CW_CONJUNCTION = "CW_CONJUNCTION"; // "and"
    public static final String CW_BECAUSE_NS = "CW_BECAUSE_NS";
    public static final String CW_BECAUSE_SN = "CW_BECAUSE_SN";
    public static final String CW_SINCE_NS = "CW_SINCE_NS";
    public static final String CW_SINCE_SN = "CW_SINCE_SN";
    public static final String CW_WHILE = "CW_WHILE";
    public static final String CW_BUT = "CW_BUT";
    // third group
    public static final String PERIOD_CW_ON_THE_OTHER_HAND_FULL_SCOPE = "PERIOD_CW_ON_THE_OTHER_HAND_FULL_SCOPE";
    public static final String PERIOD_CW_ON_THE_OTHER_HAND_SUBJECT_SCOPE = "PERIOD_CW_ON_THE_OTHER_HAND_SUBJECT_SCOPE";
    public static final String PERIOD_CW_HOWEVER_FULL_SCOPE = "PERIOD_CW_HOWEVER_FULL_SCOPE";
    public static final String PERIOD_CW_HOWEVER_SUBJECT_SCOPE = "PERIOD_CW_HOWEVER_SUBJECT_SCOPE";
    // fourth group
    public static final String PERIOD = "PERIOD";

    // ready-made operations
    public static final Operation merge_op = new Merge();
    public static final Operation relative_clause_op = new RelativeClause();
    public static final Operation with_ns_op = new WithNS();
    public static final Operation with_sn_op = new WithSN();
    public static final Operation cw_conjunction_op = new CWConjunction();
    public static final Operation cw_because_ns_op = new CWBecauseNS();
    public static final Operation cw_because_sn_op = new CWBecauseSN();
    public static final Operation cw_since_ns_op = new CWSinceNS();
    public static final Operation cw_since_sn_op = new CWSinceSN();
    public static final Operation cw_while_op = new CWWhile();
    public static final Operation cw_but_op = new CWBut();
    public static final Operation period_cw_on_the_other_hand_full_scope_op = new PeriodCWOnTheOtherHandFS();
    public static final Operation period_cw_on_the_other_hand_subject_scope_op = new PeriodCWOnTheOtherHandSS();
    public static final Operation period_cw_however_full_scope_op = new PeriodCWHoweverFS();
    public static final Operation period_cw_however_subject_scope_op = new PeriodCWHoweverSS();
    public static final Operation period_op = new Period();

    // to indicate if the target is the top plan
    boolean isTopPlan = false;

    public boolean isTopPlan(){
	return isTopPlan;
    }

    public void setTopPlan(boolean b){
	isTopPlan = b;
    }

    public void checkInvalidOperationInStructure(Aggregatable a) throws AggregationException{
	if(a instanceof AggregatedElement){
	    /** for the element **/
	    String op_name = ((AggregatedElement)a).getOperation().getOperationName();
	    if(isInvalidOperationInStructure(op_name)){
		throw new AggregationException("invalid embedding",this);
	    }
	    /** for decendants **/
	    NodeList nl = ((Element)a).getElementsByTagName(BaseElement.AGGREGATEDELEMENT_TAGNAME);
	    for(int i=0;i<nl.getLength();i++){
		AggregatedElement ae = (AggregatedElement)nl.item(i);
		String op_name2 = ae.getOperation().getOperationName();
		if(isInvalidOperationInStructure(op_name2)){
		    throw new AggregationException("invalid embedding",this);
		}
	    }
	}
    }

    public void checkInvalidOperationInStructure(Aggregatable a1, Aggregatable a2) throws AggregationException{
	checkInvalidOperationInStructure(a1);
	checkInvalidOperationInStructure(a2);
    }

    public abstract boolean isInvalidOperationInStructure(String op_name);

    public abstract AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException;

    public abstract String getOperationName();

}

