/** operation manager * */

package jsparky;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

public class OperationManager {

    private ArrayList groups;

    private Hashtable probtable;

    private Random rand;

    private Realizer realizer;

    protected ReferringExpression ref;

    protected boolean debug = false;

    protected AggregatedElement prev;

    public OperationManager() {
        groups = new ArrayList();
        probtable = new Hashtable();
        rand = new Random(System.currentTimeMillis());
        ref = new ReferringExpression();
    }

    public void setRealizer(Realizer realizer) {
        this.realizer = realizer;
    }

    public void register(OperationGroup group, int prob) {
        groups.add(group);
        probtable.put(group, new Integer(prob));
    }

    /** get random group and random operation * */
    protected Operation getRandomOperation() {
        int sum = 0;
        for (int i = 0; i < groups.size(); i++) {
            OperationGroup group = (OperationGroup) groups.get(i);
            Integer integer = (Integer) probtable.get(group);
            sum += integer.intValue();
        }
        int randnum = rand.nextInt(sum) + 1; // 0-sum (sum included)
        int total = 0;
        for (int i = 0; i < groups.size(); i++) {
            OperationGroup group = (OperationGroup) groups.get(i);
            Integer integer = (Integer) probtable.get(group);
            int prob = integer.intValue();
            if (total < randnum && randnum <= total + prob) {
                Operation operation = group.getRandomOperation();
                return operation;
            }
            total += prob;
        }
        return null;
    }

    /** get all registered operations * */
    protected List getAllOperations() {
        ArrayList lis = new ArrayList();
        for (int i = 0; i < groups.size(); i++) {
            OperationGroup group = (OperationGroup) groups.get(i);
            for (int j = 0; j < group.size(); j++) {
                Operation op = (Operation) group.get(j);
                lis.add(op);
            }
        }
        return lis;
    }

    /** set top plan flag to all operations * */
    public void setTopPlan(boolean b) {
        List ops = getAllOperations();
        for (int i = 0; i < ops.size(); i++) {
            Operation op = (Operation) ops.get(i);
            op.setTopPlan(b);
        }
    }

    /** main loop * */
    public SentencePlan aggregate(RSTPlan rst_plan) {
        SentencePlan splan = new SentencePlan(rst_plan);
        RSTElement root = (RSTElement) splan.getDocumentElement();
        setTopPlan(false);
        RSTElement target = nextTarget(splan);
        while (target != null) {
            if (target == splan.getDocumentElement()) {//top-plan
                setTopPlan(true);
            }
            aggregate(splan, target);
            target = nextTarget(splan);
        }
        postProcess(splan);
        return splan;
    }

    /** this can be overriden to do other things* */
    public void postProcess(SentencePlan splan) {
        AggregatedElement a = (AggregatedElement) splan.getDocumentElement();
        ref.process(a);
    }

    public RSTElement nextTarget(Document doc) {
        NodeList nl = doc.getElementsByTagName(BaseElement.RSTELEMENT_TAGNAME);
        for (int i = 0; i < nl.getLength(); i++) {
            RSTElement elem = (RSTElement) nl.item(i);
            NodeList nl2 = elem.getChildNodes();
            List lis = getAggregatableChildren(elem);
            if (nl2.getLength() == lis.size()) {
                return elem;
            }
        }
        return null;
    }

    /** get SentencePlanElement or PropositionElement * */
    protected List getAggregatableChildren(RSTElement parent) {
        ArrayList lis = new ArrayList();
        for (Node child = parent.getFirstChild(); child != null; child = child
                .getNextSibling()) {
            if (child instanceof Aggregatable) {
                lis.add(child);
            }
        }
        return lis;
    }

    /** 
     * Perform main aggregation. Picks random operation, while it fails. 
     * 
     *
     ***/
    protected void aggregate(SentencePlan splan, RSTElement elem) {
        List lis = getAggregatableChildren(elem);
        while (true) {
            if (lis.size() >= 2) {
                boolean success = false;
                ArrayList failed_operations = new ArrayList();
                Aggregatable a1, a2;
                if (lis.size() == 2) {
                    a1 = (Aggregatable) lis.get(0);
                    a2 = (Aggregatable) lis.get(1);
                } else {
                    Object obj[] = selectTwo(lis);
                    a1 = (Aggregatable) obj[0];
                    a2 = (Aggregatable) obj[1];
                }
                while (!success) {
                    try {
                        Operation operation = getRandomOperation();
                        if (failed_operations.contains(operation)) {
                            continue;
                        }
                        System.out.println("<<< OPERATION : "
                                + operation.getOperationName() + " >>>");
                        AggregationResult result = aggregate(operation, elem,
                                a1, a2);
                        AggregatedElement aelem = splan
                                .createAggregatedElement(result, elem,
                                        operation);
                        lis.set(lis.indexOf(a1), aelem);
                        lis.remove(a2);
                        aelem.appendChild((Node) a1);
                        aelem.appendChild((Node) a2);
                        String text = getText(aelem);
                        System.out.println("\ntext : " + text + "\n");
                        //System.out.println("<<< SUCCESS >>>");
                        success = true;
                    } catch (AggregationException e) {
                        Operation failed_op = e.getOperation();
                        System.out.println("<<< FAIL : "
                                + failed_op.getOperationName() + " >>>");
                        // e.printStackTrace();
                        failed_operations.add(failed_op);
                    }
                }
            } else if (lis.size() == 1) {
                Node parent = elem.getParentNode();
                if (parent != null) {
                    parent.replaceChild((Node) lis.get(0), elem);
                }
                break;
            }
            //System.out.println("iteration...");
        }
    }

    /** Utility Functions below * */

    // returns aggregated dss
    protected AggregationResult aggregate(Operation operation,
            RSTElement rst_elem, Aggregatable a1, Aggregatable a2)
            throws AggregationException {
        /** setup ns ordering * */
        if (Relation.isMonoNuclear(rst_elem.getRelationName())) {
            if (a1 instanceof AggregatedElement) {
                a1.setNS(Relation.SATELLITE);
            }
            if (a2 instanceof AggregatedElement) {
                a2.setNS(Relation.SATELLITE);
            }
        }
        /** debug print * */
        if (debug) {
            System.out.println(Utils.serialize(rst_elem));
            System.out.println("input1 " + Utils.serialize((Element) a1));
            System.out.println("input2 " + Utils.serialize((Element) a2));
            if (a1.getDSS() != null) {
                System.out.println(Utils.serialize(a1.getDSS()
                        .getDocumentElement()));
            }
            if (a2.getDSS() != null) {
                System.out.println(Utils.serialize(a2.getDSS()
                        .getDocumentElement()));

            }
        }
        AggregationResult result = operation.aggregate(rst_elem, a1, a2);
        if (debug) {
            if (result.getAggregatedDSS() != null) {
                System.out.println(Utils.serialize(result.getAggregatedDSS()));
            }
        }
        return result;
    }

    /** select two sequential objects * */
    protected Object[] selectTwo(List lis) {
        int len = lis.size();
        int randnum = rand.nextInt(len - 1);
        return new Object[] { lis.get(randnum), lis.get(randnum + 1) };
    }

    public void setDebug(boolean b) {
        this.debug = b;
    }

    /** get all the text under AggregatedElement * */
    public String getText(AggregatedElement elem) {
        StringWriter sw = new StringWriter();
        getText(elem, sw);
        return sw.toString().trim();
    }

    public void getText(AggregatedElement elem, StringWriter sw) {
        AggregationResult result = elem.getAggregationResult();
        if (result.hasOneSentence()) {
            String sent = realizer.realize(result.getAggregatedDSS());
            sw.write(sent + " ");
        } else {
            if (result.getFirstDSS() != null) {
                String sent = realizer.realize(result.getFirstDSS());
                sw.write(sent + " ");
            } else {
                getText((AggregatedElement) elem.getFirstChild(), sw);
            }
            if (result.getSecondDSS() != null) {
                String sent = realizer.realize(result.getSecondDSS());
                sw.write(sent + " ");
            } else {
                getText((AggregatedElement) elem.getLastChild(), sw);
            }
        }
    }
    
    /**
     * Returns a list of DSyntS, one per sentence.
     * Added by Francois Mairesse, 18/09/2006.
     * 
     * @param elem root AggregatedElement from which to build DSyntS tree
     * @return list of root DSyntS elements compatible with RealPro
     */
    public List getAggregatedDSS(AggregatedElement elem) {
        
        ArrayList dssList = new ArrayList();
        AggregationResult result = elem.getAggregationResult();
        if (result.hasOneSentence()) {
            dssList.add(result.getAggregatedDSS());
        } else {
            if (result.getFirstDSS() != null) {
                dssList.add(result.getFirstDSS());
               
            } else {
                dssList.addAll(getAggregatedDSS((AggregatedElement) elem.getFirstChild()));
            }
            if (result.getSecondDSS() != null) {
                dssList.add(result.getSecondDSS());
            } else {
                dssList.addAll(getAggregatedDSS((AggregatedElement) elem.getLastChild()));
            }
        }
        return dssList;
    }
    
    

}
