/**
 * RSTTransformer Currently handles only compare2, compare3, and recommend
 * strategies N.B. This is a very adhoc implementation of RSTTransformer
 * 
 * created by Ryuichiro Higashinaka
 * 
 * @shef 2005/05/25 2005/05/27
 *  
 */
package jsparky;

import java.util.*;
import org.w3c.dom.*;
import java.util.*;

public class RSTTransformer extends Object {

    /** global constants * */
    public static final String COMPARE2 = "compare2";

    public static final String COMPARE3 = "compare3";

    public static final String RECOMMEND = "recommend";

    public static final String[] STRATEGY_NAMES = { "compare2", "compare3",
            "recommend" };

    /**
     * Gets the strategy assigned to the relations. TODO only one relation per
     * strategy? Use dialogue act instead
     *  
     */
    public static String getStrategy(List relations) {
        if (relations.size() > 0) {
            Relation relation = (Relation) relations.get(0);
            String rel_name = relation.getRelationName();
            if (rel_name.equals(Relation.CONTRAST)) {
                return RSTTransformer.COMPARE2;
            } else if (rel_name.equals(Relation.ELABORATION)) {
                return RSTTransformer.COMPARE3;
            } else if (rel_name.equals(Relation.JUSTIFY)) {
                return RSTTransformer.RECOMMEND;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    /**
     * Creates RST plan depending on dialogue act. 
     * Applies basic text plan rules (creating infer relations) and randomizes tree.
     */
    public RSTPlan transform(List relations) throws RSTTransformException {
        RSTPlan rstplan = new RSTPlan(relations);
        String strategy = RSTTransformer.getStrategy(relations);
        if (strategy.equals(RSTTransformer.COMPARE2)) {
            rstplan.setStrategy(RSTTransformer.COMPARE2);
            rstplan = compare2(rstplan);
        } else if (strategy.equals(RSTTransformer.COMPARE3)) {
            rstplan.setStrategy(RSTTransformer.COMPARE3);
            rstplan = compare3(rstplan);
        } else if (strategy.equals(RSTTransformer.RECOMMEND)) {
            rstplan.setStrategy(RSTTransformer.RECOMMEND);
            rstplan = recommend(rstplan);
        } else {
            throw new RSTTransformException("no such strategy");
        }
        return rstplan;
    }

    /** only contrast * */
    protected RSTPlan compare2(RSTPlan rstplan) throws RSTTransformException {
        ArrayList relations = rstplan.getRelationsByName(Relation.CONTRAST);
        if (relations.size() == 1) {
            /** simply constract a tree * */
            Relation relation = (Relation) relations.get(0);
            RSTElement contrast = rstplan.createRSTElement(Relation.CONTRAST);
            ArrayList propositions = relation.getPropositions();
            for (int j = 0; j < propositions.size(); j++) {
                Proposition prop = (Proposition) propositions.get(j);
                String ns = relation.getNS(prop);
                PropositionElement propElem = rstplan.createPropositionElement(
                        prop, ns);
                contrast.appendChild(propElem);
            }
            rstplan.appendChild(contrast);
            randomOrdering(rstplan, true);
            return rstplan;
        } else if (relations.size() > 1) {
            // multiple contrast relations (compare3)
            // produces an inference of multiple contrasts (good?)
            /** buggy here * */
            ArrayList nucleusList = rstplan.getPropositions(Relation.CONTRAST,
                    Relation.NUCLEUS);
            if (!Utils.hasSingleObject(nucleusList)) {
                RSTElement infer = rstplan.createRSTElement(Relation.INFER);
                for (int i = 0; i < relations.size(); i++) {
                    Relation relation = (Relation) relations.get(i);
                    ArrayList propositions = relation.getPropositions();
                    RSTElement contrast = rstplan
                            .createRSTElement(Relation.CONTRAST);
                    for (int j = 0; j < propositions.size(); j++) {
                        Proposition prop = (Proposition) propositions.get(j);
                        String ns = relation.getNS(prop);
                        PropositionElement propElem = rstplan
                                .createPropositionElement(prop, ns);
                        contrast.appendChild(propElem);
                    }
                    infer.appendChild(contrast);
                }
                rstplan.appendChild(infer);
                //RSTPlan org = (RSTPlan)rstplan.cloneNode(true);
                //System.out.println("orgplan (rstplan): " +
                // Utils.serialize(rstplan.getDocumentElement()));
                
                // promote rstplan with 50% chance
                RSTPlan org = new RSTPlan(rstplan);
                // promote relation up (infer(contrast) --> contrast(infer))
                promote(rstplan);
                
                //System.out.println("orgplan : " +
                // Utils.serialize(org.getDocumentElement()));
                //System.out.println("2ndplan : " +
                // Utils.serialize(rstplan.getDocumentElement()));
                Random rand = new Random(System.currentTimeMillis());
                if (rand.nextInt() % 2 == 0) {
                    //System.out.println("org is rstplan");
                    rstplan = org;
                } else {
                    //System.out.println("org is not rstplan");
                }
                //System.out.println("selected : " +
                // Utils.serialize(rstplan.getDocumentElement()));
                randomOrdering(rstplan, true);
                //System.out.println("randomly ordered : " +
                // Utils.serialize(rstplan.getDocumentElement()));
                return rstplan;
            } else {
                throw new RSTTransformException(
                        "just one new nucleus propositions for compare2");
            }
        } else {
            throw new RSTTransformException("invalid compare2 strategy");
        }
    }

    /** 
     * Elaboration and then contrast 
     * Create elaboration roots and calls compare2() with the contrast relations.
     **/
    protected RSTPlan compare3(RSTPlan rstplan) throws RSTTransformException {
        
        // get nucleus from elaborations
        ArrayList nucleusList = rstplan.getPropositions(Relation.ELABORATION,
                Relation.NUCLEUS);
        if (Utils.hasSingleObject(nucleusList)) {
            RSTElement elaboration = rstplan
                    .createRSTElement(Relation.ELABORATION);
            rstplan.appendChild(elaboration);
            Proposition prop = (Proposition) nucleusList.get(0);
            PropositionElement propElem = rstplan.createPropositionElement(
                    prop, Relation.NUCLEUS);
            elaboration.appendChild(propElem);
            ArrayList contrast_relations = rstplan
                    .getRelationsByName(Relation.CONTRAST);
            // create (possibly) multi contrast RST tree
            RSTPlan new_rstplan = new RSTPlan(contrast_relations);
            new_rstplan = compare2(new_rstplan);
            // append it to rstplan
            RSTElement elem = new RSTElement(rstplan, (RSTElement) new_rstplan
                    .getDocumentElement(), true);
            elaboration.appendChild(elem);
            return rstplan;
        } else {
            throw new RSTTransformException("invalid compare3 strategy");
        }
    }

    /** 
     * only justify exists 
     * Put all satellites of the justify relation under an infer relation.
     **/
    protected RSTPlan recommend(RSTPlan rstplan) throws RSTTransformException {
        ArrayList relations = rstplan.getRelationsByName(Relation.JUSTIFY);
        if (relations.size() > 0) {
            // get the claim
            ArrayList nucleusList = rstplan.getPropositions(Relation.JUSTIFY,
                    Relation.NUCLEUS);
            if (Utils.hasSingleObject(nucleusList)) {
                ArrayList satelliteList = rstplan.getPropositions(
                        Relation.JUSTIFY, Relation.SATELLITE);
                RSTElement root = rstplan.createRSTElement(Relation.JUSTIFY);
                PropositionElement elem = rstplan.createPropositionElement(
                        (Proposition) nucleusList.get(0), Relation.NUCLEUS);
                root.appendChild(elem);
                RSTElement infer = rstplan.createRSTElement(Relation.INFER);
                for (int j = 0; j < satelliteList.size(); j++) {
                    /** make them nucleus under infer * */
                    PropositionElement propElem = rstplan
                            .createPropositionElement(
                                    (Proposition) satelliteList.get(j),
                                    Relation.NUCLEUS);
                    infer.appendChild(propElem);
                }
                root.appendChild(infer);
                rstplan.appendChild(root);
                // randomize tree
                randomOrdering(rstplan);
                return rstplan;
            } else {
                throw new RSTTransformException(
                        "more than one new nucleus propositions for recommend");
            }
        } else {
            throw new RSTTransformException("invalid recommend strategy");
        }
    }

    /** 
     * Converts a infer(contrast, ...) into a contrast(infer, ...) RST tree.
     **/
    protected void promote(RSTPlan rstplan) throws RSTTransformException {
        
        RSTElement root = (RSTElement) rstplan.getDocumentElement();
        RSTElement newroot = rstplan.createRSTElement(Relation.CONTRAST);
        String rel_name = root.getRelationName();
        if (rel_name.equals(Relation.INFER)) {
            ArrayList rel_names = new ArrayList();
            ArrayList rstElemList = new ArrayList();
            // keep track of relation names in children RSTElement
            for (Node child = root.getFirstChild(); child != null; child = child
                    .getNextSibling()) {
                if (child instanceof RSTElement) {
                    RSTElement rstElem = (RSTElement) child;
                    if (!rel_names.contains(rstElem.getRelationName())) {
                        rel_names.add(rstElem.getRelationName());
                    }
                    rstElemList.add(rstElem);
                }
            }
            // if only one relation in children, and multinuclear
            if (rel_names.size() == 1
                    && !Relation.isMonoNuclear((String) rel_names.get(0))) {
                RSTElement rstElem = (RSTElement) rstElemList.get(0);
                NodeList nl = rstElem
                        .getElementsByTagName(BaseElement.PROPOSITIONELEMENT_TAGNAME);
                int len = nl.getLength();
                // for each proposition, create new infer relation under the new root (contrast)
                for (int i = 0; i < len; i++) {
                    RSTElement infer = rstplan.createRSTElement(Relation.INFER);
                    for (int j = 0; j < rstElemList.size(); j++) {
                        RSTElement el = (RSTElement) rstElemList.get(j);
                        NodeList propList = el
                                .getElementsByTagName(BaseElement.PROPOSITIONELEMENT_TAGNAME);
                        PropositionElement prop = rstplan
                                .createPropositionElement((PropositionElement) propList
                                        .item(i));
                        infer.appendChild(prop);
                    }
                    newroot.appendChild(infer);
                    //System.out.println("midplan : " +
                    // Utils.serialize(newroot));
                }
                rstplan.removeChild(root);
                rstplan.appendChild(newroot);
            }
        } else {
            throw new RSTTransformException("cannot promote");
        }
    }

    /** random ordering * */
    protected void randomOrdering(RSTPlan rstplan) {
        randomOrdering(rstplan, false); // default
    }

    protected void randomOrdering(RSTPlan rstplan, boolean imposeOrdering) {
        //System.out.println(rstplan.getDocumentElement().getClass());
        RSTElement root = (RSTElement) rstplan.getDocumentElement();
        //System.out.println(Utils.serialize((Element)root));
        randomOrdering(root, imposeOrdering, new Hashtable());
    }

    /**
     * Changes ordering of each Element of subtree of a node, recursively. 
     * Only with infer, justify and contrast relations.
     * 
     * @param parent
     * @param imposeOrdering
     *            if <code>ordering</code> is empty, create a random one in
     *            the hashtable, otherwise use the specified ordering for
     *            shifting children.
     * @param ordering
     *            specifies mapping from old to new indexes of children array
     */
    protected void randomOrdering(RSTElement parent, boolean imposeOrdering,
            Hashtable ordering) {
        //System.out.println("randomordering : " + Utils.serialize(parent));
        //System.out.println("ordering : " + ordering);
        String rel_name = parent.getRelationName();
        //System.out.println("rel_name : " + rel_name);
        if (rel_name.equals(Relation.INFER)
                || rel_name.equals(Relation.JUSTIFY)
                || rel_name.equals(Relation.CONTRAST)) {
            NodeList nl = parent.getChildNodes();
            // list of Element children
            ArrayList lis = new ArrayList();
            /** register and temporarily removing children * */
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
                if (child.getNodeType() == Node.ELEMENT_NODE) {
                    //parent.removeChild(child);
                    lis.add(child);
                }
            }
            //System.out.println("lis : " + lis);
            while (nl.getLength() > 0) {
                parent.removeChild((Node) nl.item(0));
            }
            /** setting orders * */
            if (imposeOrdering) {
                if (!ordering.isEmpty()) {
                    //ArrayList newlis = new ArrayList(lis.size());
                    ArrayList newlis = (ArrayList) lis.clone();
                    //System.out.println("lis : " + lis);
                    for (int i = 0; i < lis.size(); i++) {
                        int j = ((Integer) ordering.get(new Integer(i)))
                                .intValue();
                        //System.out.println(i + " => " + j);
                        newlis.set(j, lis.get(i));
                    }
                    //System.out.println("newlis : " + newlis);
                    lis = newlis;
                } else {
                    // shuffle children, and put mapping of indexes in hashtable
                    ArrayList org = (ArrayList) lis.clone();
                    //System.out.println("org : " + org);
                    Utils.shuffle(lis);
                    //System.out.println("lis : " + lis);
                    ordering.clear();
                    for (int i = 0; i < org.size(); i++) {
                        for (int j = 0; j < lis.size(); j++) {
                            if (org.get(i) == lis.get(j)) {
                                ordering.put(new Integer(i), new Integer(j));
                            }
                        }
                    }
                    //System.out.println("ordering created : " + ordering);
                }
            } else {
                Utils.shuffle(lis);
            }
            /**
             * adding again in new order
             */
            for (int i = 0; i < lis.size(); i++) {
                parent.appendChild((Element) lis.get(i));
            }
            //System.out.println("result : " + Utils.serialize(parent));
            /** recursive call to randomize children * */
            Hashtable newordering = new Hashtable();
            for (int i = 0; i < lis.size(); i++) {
                /** here the list is proposition... * */
                if (lis.get(i) instanceof RSTElement) {
                    randomOrdering((RSTElement) lis.get(i), imposeOrdering,
                            newordering);
                }
            }
        }
        /** if not infer, justify or contrast, skip and try other children * */
        else {
            Hashtable newordering = new Hashtable();
            for (Node child = parent.getFirstChild(); child != null; child = child
                    .getNextSibling()) {
                if (child instanceof RSTElement) {
                    randomOrdering((RSTElement) child, imposeOrdering,
                            newordering);
                }
            }
        }
    }
}

// end of file
