/** Realizer: Realpro * */

package jsparky;

import java.io.*;
import java.util.*;

import com.cogentex.real.api.*;
import org.w3c.dom.*;

import java.lang.reflect.*;
import com.cogentex.real.parser.*;

public class Realizer {

    public static RealProMgr realproManager = new RealProMgr();

    private Hashtable dsstable;

    public Realizer() {
        //realproManager = new RealProMgr();
        realproManager.setMissingArgumentRecovery(true);
        dsstable = new Hashtable();
        //showRealProMethods();
    }

    public Realizer(String lexicon) {
        this();
        System.out.println("loading " + lexicon);
    }

    public void loadDSSFiles(String dir) {
        Vector vec = Utils.getFiles(dir, "dss");
        for (int i = 0; i < vec.size(); i++) {
            File f = (File) vec.elementAt(i);
            String path = f.getPath();
            String name = f.getName();
            if (name.endsWith(".dss")) {
                name = name.substring(0, name.length() - 4);
            }
            Document doc = realproManager.loadDSSFile(path);
            if (doc != null) {
                dsstable.put(name, doc);
            } else {
                Thread.dumpStack();
                System.out.println("ERROR: cannot load " + path);
            }
        }
    }

    protected void showRealProMethods() {
        Method[] dmethod = realproManager.getClass().getDeclaredMethods();
        System.out.println("----- all methods -------");
        for (int i = 0; i < dmethod.length; i++) {
            System.out.println(dmethod[i].toString());
        }
        System.out.println();
    }

    public Document loadDSSFile(String path) {
        Document doc = realproManager.loadDSSFile(path);
        return doc;
    }

    public Document getDSSForName(String dssname) {
        return (Document) dsstable.get(dssname);
    }

    public String realize(Document doc) {
        Document realized_doc = realproManager.realize(doc);
        if (realized_doc != null) {
            return realproManager.getSentenceString();
        } else {
            return "";
        }
    }

    public String realize(String path) {
        Document doc = null;
        StringWriter sw = new StringWriter();
        try {
            doc = realproManager.loadDSSFile(path);
            if (doc != null) {
                sw.write("xml : \n" + Utils.serialize(doc.getDocumentElement())
                        + "\n");
                doc = realproManager.realize(doc);
                String str = realproManager.toDSSString(doc
                        .getDocumentElement(), true, "");
                sw.write("dss : \n" + str + "\n");
                String sent = realproManager.getSentenceString();
                sw.write("sent : " + sent + "\n");
            } else {
                sw.write("ERROR: realization failed\n");
            }
        } catch (com.cogentex.real.parser.TokenMgrError e) {
            sw.write("ERROR: cannot load " + path + "\n");
        }
        return sw.toString();
    }

    /** realize dss string * */
    public String write_to_file(String dss_str) throws Exception {
        File tmp = File.createTempFile("realizer", ".tmp");
        tmp.deleteOnExit();
        BufferedWriter bw = new BufferedWriter(new FileWriter(tmp));
        bw.write(dss_str);
        bw.close();
        return tmp.getPath();
    }

    /** read from STDIN * */
    public void realize() {
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(
                    System.in));
            String line = "";
            while ((line = br.readLine()) != null) {
                if (!line.startsWith("DSYNTS:")) {
                    System.out.println(line);
                    continue;
                }
                try {
                    System.out.println("\n-------------");
                    System.out.println("line : " + line);
                    System.out.println();
                    String filename = write_to_file(line);
                    String str = realize(filename);
                    System.out.println(str);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            br.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String args[]) {
        Realizer r = new Realizer();
        if (args.length == 0) {
            r.realize();
        } else {
            r.realize(args[0]);
        }
    }
}
