/**
 * SpeechPlan.java
 * 
 * created by Ryuichiro Higashinaka
 * 
 * @shef 2005/05/25
 *  
 */
package jsparky;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;

public class SpeechPlan extends Object {

    private ArrayList relations;

    private ArrayList propositions;

    private Hashtable propTable;

    public SpeechPlan(String filename) {
        relations = new ArrayList();
        propositions = new ArrayList();
        propTable = new Hashtable();
        load(filename);
        System.err.println("SpeechPlan : " + filename + " loaded");
    }

    /**
     * Loads propositions and relations lists.
     * 
     * @param filename
     */
    private void load(String filename) {
        Document doc = Utils.parseDocument(filename);
        if (doc != null) {
            /** processing proposition **/
            NodeList nl = doc.getElementsByTagName("proposition");
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element) nl.item(i);
                String id = elem.getAttribute("id");
                String dssname = elem.getAttribute("dialogue_act");
                if (dssname.equals("")) {
                    continue;
                } // proposition in relation
                Proposition prop = (Proposition) propTable.get(id);
                if (prop == null) {
                    prop = new Proposition(elem);
                }
                propositions.add(prop);
                propTable.put(id, prop);
            }
            /** processing relation **/
            nl = doc.getElementsByTagName("relation");
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element) nl.item(i);
                String relation_name = elem.getAttribute("name");
                Relation rel = new Relation(relation_name);
                for (Node child = elem.getFirstChild(); child != null; child = child
                        .getNextSibling()) {
                    // add proposition in relation object
                    if (child.getNodeType() == Node.ELEMENT_NODE) {
                        Element propElem = (Element) child;
                        String ns = propElem.getAttribute("ns");
                        String id = propElem.getAttribute("id");
                        Proposition prop = (Proposition) propTable.get(id);
                        rel.addProposition(prop, ns);
                    }
                }
                relations.add(rel);
            }
        } else {
            System.err.println("ERROR: document is not created");
        }
    }

    /** 
     * For each proposition, clone DSyntS from Realizer object, and replace 
     * variables in DSyntS by each proposition's values. 
     **/
    public void bindDSS(Realizer realizer) {
        for (int i = 0; i < propositions.size(); i++) {
            Proposition prop = (Proposition) propositions.get(i);
            String dssname = prop.getDSSName();
            Document dss = realizer.getDSSForName(dssname);
            // copy dss
            if (dss != null) {
                DocumentImpl newdss = new DocumentImpl();
                newdss.appendChild(newdss.importNode(dss.getDocumentElement()
                        .cloneNode(true), true));
                prop.setDSS(newdss);
                prop.fillDSS();
                dss = prop.getDSS();
                //System.out.println("loaded : " + dssname + " : " +
                // prop.getId() + " : " +
                // Utils.serialize(newdss.getDocumentElement()));
            } else {
                System.out.println("cannot find dss for " + dssname);
            }
        }
    }

    /** getter **/
    public List getRelations() {
        return relations;
    }

    public List getPropositions() {
        return propositions;
    }

    public static void main(String args[]) {
        SpeechPlan splan = new SpeechPlan(args[0]);
    }

}
