/** utils : contains every utility **/
package jsparky;

import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.oro.text.perl.*;
import org.apache.oro.text.regex.*;
import org.apache.xml.serialize.*; 
import org.apache.xerces.parsers.*;
import org.apache.xerces.dom.*;
import org.apache.xpath.*;
import java.util.*;
import org.w3c.dom.traversal.*;
import com.cogentex.real.api.*;

/** simple utils **/
public class Utils {

    /**************************************************************
                          XML Utility
    ***************************************************************/

    public static String getText(Node node){
	return getText(node,new StringWriter());
    }
    
    public static String getText(Node node, StringWriter sw){
	for(Node child = node.getFirstChild();child != null; child = child.getNextSibling()){
	    if(child.getNodeType() == Node.ELEMENT_NODE){
		getText(child,sw);
	    }else if(child.getNodeType() == Node.TEXT_NODE){
		sw.write(child.getNodeValue());
	    }
	}
	return sw.toString();
    }

    /** parse XML document **/
    public static Document parseDocument(String filename){
	Document doc = null;
	try{
	    DOMParser parser = new DOMParser();
	    parser.parse(filename);
	    doc = parser.getDocument();
	}catch(Exception e){
	    e.printStackTrace();
	}
	return doc;
    }
    
    /** serialize **/
    public static String serialize(Document doc){
	StringWriter sw = new StringWriter();
	XMLSerializer s = new XMLSerializer();
	s.setOutputCharStream(sw);
	OutputFormat format = new OutputFormat();
	format.setIndenting(true);
	format.setOmitDocumentType(true);
	format.setOmitXMLDeclaration(true);
	s.setOutputFormat(format);
	try{
	    s.serialize(doc);
	}catch(Exception e){
	    e.printStackTrace();
	}
	String str = sw.toString();
	return str;
	//return serialize(doc.getDocumentElement());
    }

    /** serialize function **/
    public static String serialize(Element elem){
	StringWriter sw = new StringWriter();
	XMLSerializer s = new XMLSerializer();
	s.setOutputCharStream(sw);
	OutputFormat format = new OutputFormat();
	format.setIndenting(true);
	format.setOmitDocumentType(true);
	format.setOmitXMLDeclaration(true);
	s.setOutputFormat(format);
	try{
	    s.serialize(elem);
	}catch(Exception e){
	    e.printStackTrace();
	}
	String str = sw.toString();
	return str;
    }

    /** get a single specified element from a document **/
    public static Element getSingleElementByTagAndAttr(Document document, String tag, String attrname, String attrvalue){
	List lis = getElementsByTagAndAttr(document,tag,attrname,attrvalue);
	if(lis.size() ==1){
	    return (Element)lis.get(0);
	}
	return null;
    }

    /** get a single specified element from a document **/
    public static Element getSingleElementByTagAndAttr(Element el, String tag, String attrname, String attrvalue){
	List lis = getElementsByTagAndAttr(el,tag,attrname,attrvalue);
	if(lis.size() ==1){
	    return (Element)lis.get(0);
	}
	return null;
    }

    /** get first specified element under an element **/
    public static Element getFirstElementByTagAndAttr(Element el, String tag, String attrname, String attrvalue){
	List lis = getElementsByTagAndAttr(el,tag,attrname,attrvalue);
	if(lis.size() >0){
	    return (Element)lis.get(0);
	}
	return null;
    }

    /** get specified elements from a document **/
    public static List getElementsByTagAndAttr(Document document, String tag, String attrname, String attrvalue){
	ArrayList lis = new ArrayList();
	NodeList nl = document.getElementsByTagName(tag);
	for(int i=0;i<nl.getLength();i++){
	    Node node = nl.item(i);
	    if(node instanceof Element){
		Element elem = (Element)node;
		String val = elem.getAttribute(attrname);
		if(val.equals(attrvalue)){
		    lis.add(elem);
		}
	    }
	}
	return lis;
    }

    /** get specified elements from a document **/
    public static List getElementsByTagAndAttr(Element el, String tag, String attrname, String attrvalue){
	ArrayList lis = new ArrayList();
	NodeList nl = el.getElementsByTagName(tag);
	for(int i=0;i<nl.getLength();i++){
	    Node node = nl.item(i);
	    if(node instanceof Element){
		Element elem = (Element)node;
		String val = elem.getAttribute(attrname);
		if(val.equals(attrvalue)){
		    lis.add(elem);
		}
	    }
	}
	return lis;
    }

    /** getchildren **/
    public static List getChildrenByTagName(Element parent, String tagName){
	ArrayList lis = new ArrayList();
	for(Node child=parent.getFirstChild();child!=null;child=child.getNextSibling()){
	    if(child.getNodeType() == Node.ELEMENT_NODE){
		if(tagName.equals(((Element)child).getTagName())){
		    lis.add(child);
		}
	    }
	}
	return (List)lis;
    }

    /** getchildren **/
    public static List getChildrenByTagAndAttr(Element parent, String tagName, String attrname, String attrvalue){
	ArrayList lis = new ArrayList();
	for(Node child=parent.getFirstChild();child!=null;child=child.getNextSibling()){
	    if(child.getNodeType() == Node.ELEMENT_NODE){
		Element elem = (Element)child;
		if(elem.getTagName().equals(tagName) && elem.getAttribute(attrname).equals(attrvalue)){
		    lis.add(child);
		}
	    }
	}
	return (List)lis;
    }

    /** convert nodelist to a list **/
    public static List nodeListToList(NodeList nl){
	ArrayList lis = new ArrayList();
	for(int i=0;i<nl.getLength();i++){
	    lis.add(nl.item(i));
	}
	return lis;
    }

    /** copy attributes from src to target **/
    public static void copyAttributes(Element src, Element target){
	NamedNodeMap nodemap = src.getAttributes();
	for(int i=0;i<nodemap.getLength();i++){
	    Attr attr = (Attr)nodemap.item(i);
	    String name  = attr.getName();
	    String value = attr.getValue();
   	    target.setAttribute(name,value);
	}	
    }

    /** Same structure **/
    public static boolean isIdentical(Element e1, Element e2, boolean deep){
	if(deep){
	    return isIdenticalStructure(e1,e2);
	}else{
	    return isIdenticalElement(e1,e2);
	}
    }

    public static boolean isIdenticalStructure(Element e1, Element e2){
	// number of children check
	if(! isIdenticalElement(e1,e2)){
	    return false;
	}
	TreeWalker w1 = createTreeWalker(e1);
	TreeWalker w2 = createTreeWalker(e2);
	while(true){
	    Element c1 = (Element)w1.nextNode();
	    Element c2 = (Element)w2.nextNode();
	    if(c1 != null && c2 != null){
		if(! isIdenticalElement(c1,c2)){
		    return false;
		}
	    }else if(c1 == null && c2 == null){
		break;
	    }else{
		return false;
	    }
	}
	return true;
    }

    public static boolean isIdenticalElement(Element e1, Element e2){
	// tagname check
	if(! e1.getTagName().equals(e2.getTagName())){
	    return false;
	}
	// attribute check
	NamedNodeMap nodemap = e1.getAttributes();
	NamedNodeMap nodemap2 = e2.getAttributes();
	if(nodemap.getLength() != nodemap2.getLength()){
	    return false;
	}
	// attribute value check
	for(int i=0;i<nodemap.getLength();i++){
	    Attr attr = (Attr)nodemap.item(i);
	    String name  = attr.getName();
	    String value = attr.getValue();
	    if(! e2.getAttribute(name).equals(value)){
		return false;
	    }
	}
	return true;
    }

    public static TreeWalker createTreeWalker(Element elem){
	DocumentTraversal traversal = (DocumentTraversal)elem.getOwnerDocument();
	TreeWalker walker = traversal.createTreeWalker(elem,NodeFilter.SHOW_ELEMENT,null,false);
	return walker;
    }

    /**************************************************************
                          General Utility
    ***************************************************************/

    /** global static for random **/
    public static Random rand = new Random();

    /** selectOne **/
    public static Object selectOne(List lis){
	//Random rand = new Random();
	int t = rand.nextInt(lis.size());
	return lis.get(t);
    }

    /** selectTwo **/
    public static Object[] selectTwo(List lis){
	List newlis = safeShuffle(lis);
	return new Object[]{newlis.get(0), newlis.get(1)};
    }

    /** binary choice **/
    public static boolean trueOrFalse(){
	int t = rand.nextInt(2);
	if(t == 0){
	    return true;
	}else{
	    return false;
	}
    }

    /** binary choice : out of 100**/
    public static boolean trueOrFalse(int percent){
	int t = rand.nextInt(100);
	if(t < percent){
	    return true;
	}else{
	    return false;
	}
    }

    // add h2 to h1
    public static void addHashtable(Hashtable h1, Hashtable h2){
	Enumeration enum = h2.keys();
	while(enum.hasMoreElements()){
	    Object key = enum.nextElement();
	    h1.put(key,h2.get(key));
	}
    }

    // check for same keys in hashtable
    public static boolean sameKeysInHashtable(Hashtable h1, Hashtable h2){
	if(h1.size() != h2.size()){
	    return false;
	}
	Enumeration enum = h1.keys();
	while(enum.hasMoreElements()){
	    Object key = enum.nextElement();
	    if(h2.get(key) == null){
		return false;
	    }
	}
	return true;
    }

    public static boolean sameHashtable(Hashtable h1, Hashtable h2){
	if(h1.size() != h2.size()){
	    return false;
	}
	Enumeration enum = h1.keys();
	while(enum.hasMoreElements()){
	    Object key = enum.nextElement();
	    if(h2.get(key) == null){
		return false;
	    }
	    if(! h1.get(key).equals(h2.get(key))){
		return false;
	    }
	}
	return true;
    }

    /** get file list under a directory **/
    public static Vector getFiles(String dir, String ext){
	File[] files = new File(dir).listFiles();
	Vector fileList = new Vector();
	for (int i=0;i<files.length;i++){
	    if(files[i].isFile() && files[i].toString().endsWith(ext)){
		fileList.addElement(files[i]);
	    }
	} 
	return fileList;
    }

    /** shuffle **/
    public static void shuffle(List lis){
	for(int i=0;i<lis.size();i++){
	    int randnum = rand.nextInt(lis.size());
	    Object tmp = lis.get(i);
	    lis.set(i,lis.get(randnum));
	    lis.set(randnum,tmp);
	}
    }

    /** shuffle safe **/
    public static List safeShuffle(List lis){
	ArrayList newlis = new ArrayList(lis);
        shuffle(newlis);
	return newlis;
    }

    /** has only one elements in the list **/
    public static boolean hasSingleObject(List lis){
	ArrayList newlis = new ArrayList();
	for(int i=0;i<lis.size();i++){
	    Object obj = lis.get(i);
	    if(! newlis.contains(obj)){
		newlis.add(obj);
	    }
	}
	if(newlis.size() == 1){
	    return true;
	}
	return false;
    }

    /** count object in a list **/
    public static int countItem(List lis, Object target){
	int count = 0;
	for(int i=0;i<lis.size();i++){
	    Object obj = lis.get(i);
	    if(obj.equals(target)){
		count++;
	    }
	}
	return count;
    }

    /** join (as in perl) **/
    public static String join(List lis, String delim){
	StringWriter sw = new StringWriter();
	if(lis.size()>0){
	    Object first_obj = lis.get(0);
	    sw.write(first_obj.toString());
	    for(int i=1;i<lis.size();i++){
		sw.write(delim);
		Object obj = lis.get(i);
		sw.write(obj.toString());
	    }
	}
	return sw.toString();
    }

    /**************************************************************
                          DSS Utility
    ***************************************************************/

    /** get propositions under aggregatable **/
    public static List getPropositionList(Aggregatable a){
	ArrayList lis = new ArrayList();
	if(a instanceof AggregatedElement){
	    AggregatedElement elem = (AggregatedElement)a;
	    NodeList nl = elem.getElementsByTagName(BaseElement.PROPOSITIONELEMENT_TAGNAME);
	    for(int i=0;i<nl.getLength();i++){
		PropositionElement prop_elem = (PropositionElement)nl.item(i);
		Proposition prop = prop_elem.getProposition();
		lis.add(prop);
	    }
	}else{
	    PropositionElement prop_elem = (PropositionElement)a;
	    Proposition prop = prop_elem.getProposition();
	    lis.add(prop);
	}
	return lis;
    }

    /** get dss under aggregatable **/
    public static List getAllDSS(Aggregatable a){
	if(a instanceof AggregatedElement){
	    return getAllDSS((AggregatedElement)a);
	}else{
	    PropositionElement prop_elem = (PropositionElement)a;
	    List lis = new ArrayList();
	    lis.add(prop_elem.getProposition().getDSS());
	    return (List)lis;
	}
    }

    public static List getAllDSS(SentencePlan splan){
	return getAllDSS((AggregatedElement)splan.getDocumentElement());
    }

    /** process all dss under an aggregated element **/
    public static List getAllDSS(AggregatedElement elem){
	ArrayList lis = new ArrayList();
	getAllDSS(elem,lis);
	return lis;
    }

    public static void getAllDSS(AggregatedElement elem, List lis){
	AggregationResult result = elem.getAggregationResult();
	if(result.hasOneSentence()){
	    lis.add(result.getAggregatedDSS());
	}else{
	    if(result.getFirstDSS() != null){
		lis.add(result.getFirstDSS());
	    }else{
		getAllDSS((AggregatedElement)elem.getFirstChild(),lis);
	    }
	    if(result.getSecondDSS() != null){
		lis.add(result.getSecondDSS());
	    }else{
		getAllDSS((AggregatedElement)elem.getLastChild(),lis);
	    }
	}
    }

    /** process all dss under an aggregated element with an associated proposition list **/
    public static Hashtable getAllDSSHash(AggregatedElement elem){
	Hashtable hash = new Hashtable();
	getAllDSSHash(elem,hash);
	return hash;
    }

    public static void getAllDSSHash(AggregatedElement elem, Hashtable hash){
	AggregationResult result = elem.getAggregationResult();
	if(result.hasOneSentence()){
	    hash.put(result.getAggregatedDSS(),getPropositionList(elem));
	}else{
	    if(result.getFirstDSS() != null){
		hash.put(result.getFirstDSS(),getPropositionList((Aggregatable)elem.getFirstChild()));
	    }else{
		getAllDSSHash((AggregatedElement)elem.getFirstChild(),hash);
	    }
	    if(result.getSecondDSS() != null){
		hash.put(result.getSecondDSS(),getPropositionList((Aggregatable)elem.getLastChild()));
	    }else{
		getAllDSSHash((AggregatedElement)elem.getLastChild(),hash);
	    }
	}
    }

    // some has <rightmost_comma> tags
    public static String process_rightmost_comma(String text){
	String str = text.replaceAll("<rightmost_comma>","");
	str = str.replaceAll("</rightmost_comma>",",");
	String first_letter = str.substring(0,1);
	first_letter = first_letter.toUpperCase();
	String newtext = first_letter + str.substring(1);
	return newtext;
    }
   
}

// end of file
