/** 
    CueWord Conjunction Operation
    first created by Ryuichiro higashinaka 
    2005/05/31
**/

package jsparky.operations;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;
import com.cogentex.real.api.*;

public class CWConjunction extends Operation {

    public boolean isInvalidOperationInStructure(String op_name){
	if(op_name.startsWith("PERIOD") || op_name.startsWith("RELATIVE_CLAUSE")){
	    return true;
	}
	return false;
    }

    public AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException {

	/** check structure **/
	checkInvalidOperationInStructure(a1,a2);

	String relation = rst_elem.getRelationName();
	if(! (relation.equals(Relation.INFER) || relation.equals(Relation.CONTRAST))){
	    throw new AggregationException("Not infer or contrast",this);
	}
	Document dss1 = a1.getDSS();
	Document dss2 = a2.getDSS();
	Element root1 = dss1.getDocumentElement();
	Element root2 = dss2.getDocumentElement();
	Element pred1 = (Element)root1.getFirstChild();
	Element pred2 = (Element)root2.getFirstChild();
	//
	// no a verb (2005/06/15)
	if(! pred1.getAttribute("class").equals(pred2.getAttribute("class"))){
	    throw new AggregationException("joining different class of predicates",this);
	}

	// referringexpression(root1, root2)
	List op1 = getPreviousOperations(a1);
	List op2 = getPreviousOperations(a2);
	int count1 = Utils.countItem(op1, Operation.CW_BUT) + Utils.countItem(op1, Operation.CW_WHILE);
	int count2 = Utils.countItem(op2, Operation.CW_BUT) + Utils.countItem(op2, Operation.CW_WHILE);
	List and1 = findAND(pred1);
	List and2 = findAND(pred2);
	if(count1 > 1 || count2 > 1){
	    throw new AggregationException("more than one but/while found",this);
	}
	else if(count1 == 1 && count2 == 0){
	    throw new AggregationException("but/while found in one of the argument",this);
	}
	else if(count1 == 0 && count2 == 1){
	    throw new AggregationException("but/while found in one of the argument",this);
	}
	else if(and1.size() == 0 && and2.size() == 0){
	    pred2.setAttribute("rel","II");
	    Element elem = createAND(dss1);
	    elem.appendChild(dss1.importNode(pred2,true));
	    pred1.appendChild(elem);
	}
	else if(! isTopPlan()){
	    Element last_pred = pred1;
	    for(int i=0;i<and1.size();i++){
		Element and_el = (Element)and1.get(i);
		and_el.setAttribute("punct","replace_by_comma");
		last_pred = (Element)and_el.getFirstChild();
	    }
	    for(int i=0;i<and2.size();i++){
		Element and_el = (Element)and2.get(i);
		and_el.setAttribute("punct","replace_by_comma");
	    }
	    pred2.setAttribute("rel","II");
	    Element elem = createAND(dss1);
	    elem.appendChild(dss1.importNode(pred2,true));
	    last_pred.appendChild(elem);
	    
	}
	else{
	    throw new AggregationException("no condition matched",this);
	}
	Document newdss = new DocumentImpl();
	newdss.appendChild(newdss.importNode(root1,true));
	return new AggregationResult(newdss,dss1,dss2);
    }

    public List getPreviousOperations(Aggregatable a){
	ArrayList lis = new ArrayList();
	if(a instanceof AggregatedElement){
	    lis.add(((AggregatedElement)a).getOperation().getOperationName());
	    NodeList nl = ((Element)a).getElementsByTagName(BaseElement.AGGREGATEDELEMENT_TAGNAME);
	    for(int i=0;i<nl.getLength();i++){
		AggregatedElement elem = (AggregatedElement)nl.item(i);
		String op_name = elem.getOperation().getOperationName();
		lis.add(op_name);
	    }
	}
	return lis;
    }

    // elem is pred1
    public List findAND(Element elem){
	ArrayList lis = new ArrayList();
	NodeList nl = elem.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element el = (Element)nl.item(i);
	    if((el.getAttribute("lexeme").startsWith("AND")||el.getAttribute("lexeme").startsWith("and") 
		||el.getAttribute("lexeme").startsWith("but"))
	       && el.getAttribute("rel").equals("COORD")){
		// fixed here 2005/06/15
		Element pred = (Element)el.getFirstChild();
		// this was verb
		if(pred != null && pred.getAttribute("class").equals(elem.getAttribute("class"))){
		    lis.add(el);
		}
		//lis.add(el);
	    }
	}
	return lis;
    }

    /**
       public List findAND(Element elem){
	ArrayList lis = new ArrayList();
	NodeList nl = elem.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element el = (Element)nl.item(i);
	    if((el.getAttribute("lexeme").startsWith("AND")||el.getAttribute("lexeme").startsWith("and") 
		||el.getAttribute("lexeme").startsWith("but"))
	       && el.getAttribute("rel").equals("COORD")){
		lis.add(el);
	    }
	}
	return lis;
	}
    **/

    public Element createAND(Document doc){
	Element elem = doc.createElement(RealProMgr.TAG_DSYNTNODE);
	elem.setAttribute("lexeme","AND");
	elem.setAttribute("rel","COORD");
	return elem;
    }

    /** each operation has to return its name **/
    public String getOperationName(){
	return Operation.CW_CONJUNCTION;
    }

}
