/** 
    Merge Operation 
    first created by Ryuichiro higashinaka 
    2005/05/31
**/

package jsparky.operations;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;
import com.cogentex.real.api.*;

public class Merge extends Operation {

    public boolean isInvalidOperationInStructure(String op_name){
	if(op_name.startsWith("PERIOD") || op_name.startsWith("CW") || op_name.startsWith("WITH")){
	    return true;
	}
	return false;
    }

    public AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException {

	checkInvalidOperationInStructure(a1,a2);
	
	String relation = rst_elem.getRelationName();
	if(! (relation.equals(Relation.INFER) || relation.equals(Relation.CONTRAST))){
	    throw new AggregationException("not infer or contrast",this);
	}
	Document dss1 = a1.getDSS();
	Document dss2 = a2.getDSS();
	Element root1 = dss1.getDocumentElement();
	Element root2 = dss2.getDocumentElement();
	Element pred1 = (Element)root1.getFirstChild();
	Element pred2 = (Element)root2.getFirstChild();

	if(! pred1.getAttribute("lexeme").equals(pred2.getAttribute("lexeme"))){
	    throw new AggregationException("not same verb",this);
	}
	List lis1 = Utils.getChildrenByTagName(pred1,RealProMgr.TAG_DSYNTNODE);
	List lis2 = Utils.getChildrenByTagName(pred2,RealProMgr.TAG_DSYNTNODE);
	if(lis1.size() != lis2.size()){
	    throw new AggregationException("children count do not match",this);
	}
	int structure_match = 0;
	for(int i=0;i<lis1.size();i++){
	    Element e1 = (Element)lis1.get(i);
	    Element e2 = (Element)lis2.get(i);
	    if(Utils.isIdentical(e1,e2,true)){
		structure_match++;
	    }else{
		e2.setAttribute("rel","II");
		List lis = Utils.getElementsByTagAndAttr(e1,RealProMgr.TAG_DSYNTNODE,"lexeme","AND2");
		Element last_pred = e1;
		for(int j=0;j<lis.size();j++){
		    Element el = (Element)lis.get(j);
		    el.setAttribute("punct","replace_by_comma");
		    last_pred = (Element)el.getFirstChild();
		}
		Element elem = dss1.createElement(RealProMgr.TAG_DSYNTNODE);
		elem.setAttribute("lexeme","AND2");
		elem.setAttribute("rel","COORD");
		elem.appendChild(dss1.importNode(e2,true));
		last_pred.appendChild(elem);

		/**  in case of rel=I change rel=II to number:pl**/
		if(e1.getAttribute("rel").equals("I")){
		    Element pred1relII = Utils.getSingleElementByTagAndAttr(pred1,RealProMgr.TAG_DSYNTNODE,"rel","II");
		    if(pred1relII != null){
			pred1relII.setAttribute("number","pl");
		    }
		}
	    }
	}
	if(structure_match > 0){
	    Document newdss = new DocumentImpl();
	    newdss.appendChild(newdss.importNode(root1,true));
	    return new AggregationResult(newdss,dss1,dss2);
	}else{
	    throw new AggregationException("no match",this);
	}
    }

    /** each operation has to return its name **/
    public String getOperationName(){
	return Operation.MERGE;
    }

}
