/** 
    Period CueWord However Subject Scope Operation
    first created by Ryuichiro higashinaka 
    2005/06/01
**/

package jsparky.operations;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.dom.*;
import com.cogentex.real.api.*;

public class PeriodCWHoweverSS extends Operation {

    public boolean isInvalidOperationInStructure(String op_name){
	if(op_name.startsWith("PERIOD") || op_name.startsWith("CW")){
	    return true;
	}
	return false;
    }

    public AggregationResult aggregate(RSTElement rst_elem, Aggregatable a1, Aggregatable a2) throws AggregationException {

	/** check structure **/
	checkInvalidOperationInStructure(a1,a2);

	String relation = rst_elem.getRelationName();
	if(! relation.equals(Relation.CONTRAST)){
	    throw new AggregationException("not contrast",this);
	}
	Document dss1 = a1.getDSS();
	Document dss2 = a2.getDSS();
	Element root1 = dss1.getDocumentElement();
	Element root2 = dss2.getDocumentElement();
	Element pred1 = (Element)root1.getFirstChild();
	Element pred2 = (Element)root2.getFirstChild();
	
	List pn1 = findLexemeByClass(pred1,"proper_noun");
	List pn2 = findLexemeByClass(pred2,"proper_noun");
	if(! Utils.hasSingleObject(pn1) || ! Utils.hasSingleObject(pn2)){
	    throw new AggregationException("multple proper nouns",this);	    
	}

	List adj1 = findLexemeByClass(pred1,"adjective");
	List adj2 = findLexemeByClass(pred2,"adjective");
	for(int i=0;i<adj1.size();i++){
	    for(int j=0;j<adj2.size();j++){
		if(adj1.get(i).equals(adj2.get(j))){
		    throw new AggregationException("same adjectives",this);	    
		}
	    }
	}

	List num1 = findLexemeByClass(pred1,"numeral");
	List num2 = findLexemeByClass(pred2,"numeral");
	for(int i=0;i<num1.size();i++){
	    for(int j=0;j<num2.size();j++){
		if(num1.get(i).equals(num2.get(j))){
		    throw new AggregationException("same numerals",this);	    
		}
	    }
	}

	// whlie element
	Element elem = dss2.createElement(RealProMgr.TAG_DSYNTNODE);
	elem.setAttribute("lexeme","however");
	elem.setAttribute("rel","ATTR");
	elem.setAttribute("class","adverb");
	elem.setAttribute("punct","comma");
	elem.setAttribute("scope","between");
	pred2.appendChild(elem);

	return new AggregationResult(null,dss1,dss2);

    }

    // find specific "class:proper_noun" 
    public List findLexemeByClass(Element elem, String cls){
	ArrayList lis = new ArrayList();
	NodeList nl = elem.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element el = (Element)nl.item(i);
	    if(el.getAttribute("class").equals(cls)){
		String lexeme = el.getAttribute("lexeme");
		lis.add(lexeme);
	    }
	}
	return lis;
    }

    /** each operation has to return its name **/
    public String getOperationName(){
	return Operation.PERIOD_CW_HOWEVER_SUBJECT_SCOPE;
    }

}
