/** Review Manager **/
// holds review dss and find the necessary dss for a content
//
// by Ryuichiro higashinaka 2005/06/08
//                          2005/06/10

package jsparky.review;

import jsparky.*;
import java.util.*;
import org.w3c.dom.*;
import com.cogentex.real.api.*;
import java.io.*;
import org.apache.xerces.dom.*;

public class ReviewManager extends Object {

    /** content -> lis of dss hash **/
    Hashtable dsstable;
    Realizer realizer; // realizer to load dss

    // void constructor
    public ReviewManager(){
	dsstable = new Hashtable();
    }

    public void setRealizer(Realizer realizer){
	this.realizer = realizer;
    }

    /** load user review dss **/
    public void loadDSS(String dir){
	Vector vec = Utils.getFiles(dir,"dss");
	if(vec != null){
	    System.out.println("[loadDSS] loading " + vec.size() + " dss"); 
	    int count = 0;
	    for(int i=0;i<vec.size();i++){
		File f = (File)vec.elementAt(i);
		String path = f.getPath();
		String name = f.getName();
		System.out.println("path : " + path);
		if(name.endsWith(".dss")){
		    name = name.substring(0,name.length()-4);
		}
		try{
		    Content content = new Content(name);
		    //System.out.println("adding " + content);
		    Document doc = realizer.loadDSSFile(path);
		    if(doc != null){
			markSource(doc,name);//mark what part is coming from where
			setTense(doc); 
			setLowerCase(doc);
			dsstable.put(content,doc);
			count++;
		    }else{
			System.out.println("ERROR: cannot load " + name);
		    }
		}catch(ContentException ce){
		    ce.printStackTrace();
		}
	    }
	    System.out.println("[loadDSS] loadded " + count + " dss"); 
	}
    }

    /** utility to get dss **/
    public Document getDSSForContent(Content c, boolean strict) throws DSSNotFoundException {
	try{
	    List lis = getDSSList(c, strict);
	    Document dss = (Document)Utils.selectOne(lis);
	    DocumentImpl newdss = new DocumentImpl();
	    Element newroot = (Element)newdss.importNode(dss.getDocumentElement().cloneNode(true),true);
	    newdss.appendChild(newroot);
	    return newdss;
	}catch(Exception e){
	    throw new DSSNotFoundException("cannot find dss");
	}
    }

    public Document cloneDSS(Document dss){
	DocumentImpl newdss = new DocumentImpl();
	Element newroot = (Element)newdss.importNode(dss.getDocumentElement().cloneNode(true),true);
	newdss.appendChild(newroot);
	return newdss;
    }

    /** getDSSForAggregatable **/
    // addself (2005/06/15)
    public Document getDSSForAggregatable(Aggregatable a, boolean strict) throws DSSNotFoundException{
	ArrayList lis = new ArrayList();
	addPropositionElement(a,lis);
	Content content = new Content();
	for(int i=0;i<lis.size();i++){
	    Proposition prop = ((PropositionElement)lis.get(i)).getProposition();
	    Content c = new Content(prop);
	    try{
		content.addContent(c);
	    }catch(ContentException ce){
		ce.printStackTrace();
		throw new DSSNotFoundException("error in addcontent");
	    }
	}
	System.out.println("looking for " + content);
	Document dss = getDSSForContent(content, strict);
	fillDSSWithContent(dss,content);
	return dss;
    }

    public Document getDSSForAggregatable(Aggregatable a1, Aggregatable a2, boolean strict) throws DSSNotFoundException {
	ArrayList lis = new ArrayList();
	addPropositionElement(a1,lis);
	addPropositionElement(a2,lis);
	Content content = new Content();
	for(int i=0;i<lis.size();i++){
	    Proposition prop = ((PropositionElement)lis.get(i)).getProposition();
	    Content c = new Content(prop);
	    try{
		content.addContent(c);
	    }catch(ContentException ce){
		ce.printStackTrace();
		throw new DSSNotFoundException("error in addcontent");
	    }
	}
	System.out.println("looking for " + content);
	Document dss = getDSSForContent(content, strict);
	fillDSSWithContent(dss,content);
	return dss;
    }

        /** fill dss with content **/
    public void fillDSSWithContent(Document dss, Content c){
	String restaurantname = c.getInfo(Content.RESTAURANT_NAME);
	String location = c.getInfo(Content.LOCATION);
	String foodtype = c.getInfo(Content.FOOD_TYPE);
	NodeList nl = dss.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element elem = (Element)nl.item(i);
	    if(elem.getAttribute("lexeme").equalsIgnoreCase(Content.RESTAURANT_NAME)){
		if(restaurantname != null){
		    elem.setAttribute("lexeme",restaurantname);
		    elem.setAttribute("restaurant_name","true");
		}
	    }
	    else if(elem.getAttribute("lexeme").equalsIgnoreCase(Content.LOCATION)){
		if(location != null){
		    elem.setAttribute("lexeme",location);
		}
	    }
	    else if(elem.getAttribute("lexeme").equalsIgnoreCase(Content.PLACE_NAME)){
		if(location != null){
		    elem.setAttribute("lexeme",location);
		}
	    }
	    else if(elem.getAttribute("lexeme").equalsIgnoreCase(Content.FOOD_TYPE)){
		if(foodtype != null){
		    elem.setAttribute("lexeme",foodtype);
		}
	    }
	}
    }

    /** find propositions under an aggregatable node **/
    public void addPropositionElement(Aggregatable a, List lis){
	if(a instanceof PropositionElement){
	    lis.add(a);
	}else{
	    AggregatedElement elem = (AggregatedElement)a;
	    NodeList nl = elem.getElementsByTagName(BaseElement.PROPOSITIONELEMENT_TAGNAME);
	    for(int i=0;i<nl.getLength();i++){
		PropositionElement prop_elem = (PropositionElement)nl.item(i);
		lis.add(prop_elem);
	    }
	}
    }

    /** get list of contents of a rst_plan (possibly useful for relaxed match **/
    public List getContentList(RSTElement rst_elem){
	ArrayList lis = new ArrayList();
	Document doc = rst_elem.getOwnerDocument();
	NodeList nl = doc.getElementsByTagName(BaseElement.PROPOSITIONELEMENT_TAGNAME);
	for(int i=0;i<nl.getLength();i++){
	    PropositionElement prop_elem = (PropositionElement)nl.item(i);
	    Proposition prop = prop_elem.getProposition();
	    Content c = new Content(prop);
	    lis.add(c);
	}
	return lis;
    }

    /** past tense to present tenst **/
    public void setTense(Document dss){
	Element root = dss.getDocumentElement();
	setTense(root);
    }
    
    public void setTense(Element parent){
	for(Node child=parent.getFirstChild();child!=null;child=child.getNextSibling()){
	    if(child.getNodeType() == Node.ELEMENT_NODE){
		Element elem = (Element)child;
		if(! elem.getAttribute("tense").equals("")){
		    elem.setAttribute("tense","pres");
		}
		setTense(elem);
	    }
	}
    }

    /** past tense to present tenst **/
    public void markSource(Document dss, String src){
	NodeList nl = dss.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element elem = (Element)nl.item(i);
	    elem.setAttribute("src",src);
	}
    }

    /** past lexeme to lowercase in some cases **/
    public void setLowerCase(Document dss){
	Element root = dss.getDocumentElement();
	setLowerCase(root);
    }
    
    public void setLowerCase(Element parent){
	for(Node child=parent.getFirstChild();child!=null;child=child.getNextSibling()){
	    if(child.getNodeType() == Node.ELEMENT_NODE){
		Element elem = (Element)child;
		if(! elem.getAttribute("class").equals("proper_noun")){
		    elem.setAttribute("lexeme",elem.getAttribute("lexeme").toLowerCase());
		}else{
		    // exceptional cases
		    if(elem.getAttribute("lexeme").equalsIgnoreCase("food") ||
		       elem.getAttribute("lexeme").equalsIgnoreCase("waitstaff") ||
		       elem.getAttribute("lexeme").equalsIgnoreCase("atmosphere") ||
		       elem.getAttribute("lexeme").equalsIgnoreCase("service")||
		       elem.getAttribute("lexeme").equalsIgnoreCase("value")){
			elem.setAttribute("lexeme",elem.getAttribute("lexeme").toLowerCase());
		    }
		}
		setLowerCase(elem);
	    }
	}
    }

    // find matching list of dss
    public List getDSSList(Content c, boolean strict){
	ArrayList lis = new ArrayList();
	Enumeration enum = dsstable.keys();
	while(enum.hasMoreElements()){
	    Content content = (Content)enum.nextElement();
	    if(match(c,content,strict)){
		lis.add(dsstable.get(content));
	    }
	}
	return lis;
    }

    /** strict means all the content should match; else comparison without the restaurant name **/
    // finding dss does not have to match named entities (they are filled later) 2005/06/14
    public boolean match(Content c1, Content c2, boolean strict){
	if(strict){
	    return (Utils.sameHashtable(c1.getScores(), c2.getScores())
		    && Utils.sameHashtable(c1.getInfo(), c2.getInfo()));
	}else{
	    // remove restaurant name for comparison
	    Hashtable info1 = new Hashtable(c1.getInfo());
	    Hashtable info2 = new Hashtable(c2.getInfo());
	    info1.remove(Content.RESTAURANT_NAME);
	    info2.remove(Content.RESTAURANT_NAME);
	    return (Utils.sameHashtable(c1.getScores(), c2.getScores())
		    //&& Utils.sameHashtable(info1, info2));
		    && Utils.sameKeysInHashtable(info1,info2));
	}
    }

    /** strict means all the content should match; else comparison without the restaurant name **/
    /**
    public boolean match(Content c1, Content c2, boolean strict){
	if(strict){
	    return (Utils.sameHashtable(c1.getScores(), c2.getScores())
		    && Utils.sameHashtable(c1.getInfo(), c2.getInfo()));
	}else{
	    // remove restaurant name for comparison
	    Hashtable info1 = new Hashtable(c1.getInfo());
	    Hashtable info2 = new Hashtable(c2.getInfo());
	    info1.remove(Content.RESTAURANT_NAME);
	    info2.remove(Content.RESTAURANT_NAME);
	    return (Utils.sameHashtable(c1.getScores(), c2.getScores())
		    && Utils.sameHashtable(info1, info2));
	}
    }
    **/

    /** has modified **/
    // for all dss under the splan
    public boolean hasReviewElement(AggregatedElement aelem){
	List lis = Utils.getAllDSS(aelem);
	for(int i=0;i<lis.size();i++){
	    if(hasReviewElement((Document)lis.get(i))){
		return true;
	    }
	}
	return false;
    }

    public boolean hasReviewElement(Document dss){
	NodeList nl = dss.getElementsByTagName(RealProMgr.TAG_DSYNTNODE);
	for(int i=0;i<nl.getLength();i++){
	    Element elem = (Element)nl.item(i);	
	    if(! elem.getAttribute("src").equals("") ||
	       elem.getAttribute("swap").equals("true")){
		return true;
	    }
	}
	return false;
    }

}
